;parcal.asm - bring the STROBE line low, wait 50ms, then send a positive pulse
;and wait for the BUSY line to become active.  (This way the rising edge of the
;strobe signal can be used as a trigger for external hardware, if desired.)
;Then count the number of iterations it takes for the BUSY line to deactivate.
;Copyright (c) 2000 Matthew Belmonte
;
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
TITLE	  PARCAL
CODESG	  SEGMENT PARA 'Code'
	  ASSUME CS:CODESG,DS:CODESG,SS:CODESG,ES:CODESG
	  ORG 100H		;START AT END OF PSP
MAIN:	  MOV CX,DS		;GET PARALLEL PORT ADDRESS FROM BIOS SEGMENT
	  MOV DS,0040H
	  MOV DX,[08H]
	  MOV DS,CX
	  MOV AL,0FFH		;DRIVE OUTPUTS HIGH TO POWER THE INTERFACE BOX
	  OUT DX,AL
	  ADD DX,2		;POINT TO CONTROL REGISTER
	  IN AL,DX
	  AND AL,0BFH
	  MOV BL,AL
	  OR AL,01H
	  OUT DX,AL		;BRING STROBE LOW
	  IN AL,61H
	  AND AL,10H
	  MOV CX,3317
LOOP0:	  MOV AH,AL		;WAIT 50ms
LOOP1:	  IN AL,61H
	  AND AL,10H
	  CMP AL,AH
	  JE LOOP1
	  LOOP LOOP0
	  MOV AL,BL
	  PUSHF			;SAVE FLAGS AND MASK INTERRUPTS
	  CLI
	  OUT DX,AL		;BRING STROBE HIGH
	  DEC DX		;POINT TO STATUS REGISTER
WAIT0:	  IN AL,DX
	  TEST AL,80H
	  JNE WAIT0
WAIT1:	  INC CX		;MEASURE HOLD TIME
	  JE OVERFLOW
	  IN AL,DX
	  TEST AL,80H
	  JE WAIT1
OVERFLOW: POPF			;RESTORE INTERRUPT MASK
	  MOV AX,CX
	  CALL PRNTNUM		;PRINT HOLD TIME
	  MOV AX,4C00H
	  INT 21H
PRNTNUM:  MOV CX,10000		;INITIAL DIVISOR IS 10^4
PRNTDIG:  MOV DX,0		;HIGH BITS OF DIVIDEND ARE ZERO
	  DIV CX		;QUOTIENT IN AL, ZERO IN AH, REMAINDER IN DX
	  PUSH DX		;SAVE REMAINDER
	  PUSH CX		;SAVE DIVISOR
	  ADD AL,'0'		;MAKE THE QUOTIENT INTO A DIGIT
	  DEC SP		;ALLOCATE A CHARACTER BUFFER FROM THE STACK
	  MOV BX,SP		;POINT TO IT
	  MOV [BX],AL		;PUT DIGIT INTO BUFFER
	  MOV AH,40H		;OUTPUT IT
	  MOV DX,BX
	  MOV BX,1		;TO FILE HANDLE 1 (STDOUT)
	  MOV CX,1		;AS A SINGLE CHARACTER
	  INT 21H
	  INC SP		;FREE BUFFER
	  POP AX		;RESTORE DIVISOR INTO AX
	  MOV DX,0
	  MOV CX,10		;DIVIDE IT BY TEN
	  DIV CX		;NEW DIVISOR IN AX, 0 (OR 1, ON LAST LOOP) IN DX
	  MOV CX,AX		;MOVE DIVISOR BACK INTO CX
	  POP AX		;RESTORE REMAINDER INTO AX
	  AND CX,CX
	  JNE PRNTDIG		;LOOP TILL DIVISOR IS ZERO
	  MOV AH,40H
	  LEA DX,CRLF
	  MOV BX,1
	  MOV CX,2
	  INT 21H
	  RET
CRLF	  DB 0DH,0AH
	  ENDP
CODESG	  ENDS
	  END  MAIN
